# -*- coding: utf-8 -*-
"""
@author: danjr
"""

import matplotlib.pyplot as plt
import numpy as np
import matplotlib
import copy
import pandas as pd

# update this with the location of the main directory
folder_base = r'L:\DWWE\analysis\dan\trajectories_paper_shared_data\\'


df = pd.read_csv(folder_base+r'FIGURE6_horizontal_transport_data.csv')

# map wave slope ak to color
cmap_ak = copy.copy(plt.get_cmap('cividis'))
cmap_ak.set_under('gray')
norm_ak = matplotlib.colors.Normalize(vmin=0.15, vmax=0.5)
m_ak = matplotlib.cm.ScalarMappable(norm=norm_ak, cmap=cmap_ak)
def get_color_ak(ak):
    return m_ak.to_rgba(ak)

# function to get pdf of a quantity and return bin centers for plotting
def get_hist(x,bins=21,**kwargs):
    y,x = np.histogram(x,bins=bins,density=True,**kwargs)
    x = x[:-1] + np.diff(x)/2
    return x,y

fig,axs = plt.subplots(1,2,figsize=(9,4))

'''
Experimental values of nondim transport velocity vs nondim depth
'''

# compute dimensionless quantities
df['ak'] = df['a_m'] * df['k_pm']
df['vx_by_cp'] = df['mean_vx_mps'] / df['c_p_mps']
df['zk'] = df['mean_z_m'] * df['k_pm']
df['ux_SD'] = (df['ak'])**2 * df['c_p_mps'] * np.exp(2*df['zk'])
df['vx_by_uxSD'] = df['mean_vx_mps'] / df['ux_SD']

# plot
ax = axs[0]
c = ax.scatter(df['vx_by_cp'],df['zk'],c=df['ak'],cmap=cmap_ak,norm=norm_ak)

'''
Stokes drifts and enhanced Lagrangian drifts for bubbles
'''

# parameters, all in standard SI units
nu = 8.9e-7
g = 9.81
C_M = 0.5
rho_fluid = 1000
mu = nu*rho_fluid
rho_particle = 0
sigma = 0.0728
mu_a = 1.82e-5 
gamma = rho_particle / rho_fluid # rho_particle / rho_fluid
beta = (1+C_M) / (gamma+C_M)

# properties of the wave field
f_p = 1.8 # Hz
omega = 2*np.pi*f_p
k_p = omega**2 / g
c_p = omega/k_p

def rise_speed_Clift_contaminated(d,):    
    # Rise speed correlation for a contaminated bubble from Clift 1978
    mu_w = 0.0009 # kg/m s, viscosity of water in Braida's experiments    
    d = np.atleast_1d(d)
    Eo = g * rho_fluid * d**2 / sigma
    M = g * mu**4 * rho_fluid / (rho_fluid**2 * sigma**3)
    H = (4./3) * Eo * M**(-0.149) * (mu/mu_w)**(-0.14)    
    J = np.zeros_like(H) * np.nan    
    J[(H>2)*(H<=59.3)] = 0.94 * H[(H>2)*(H<=59.3)]**0.757    
    J[H>59.3] = 3.42 * H[H>59.3]**0.441    
    w = mu / (rho_fluid*d) * M**-0.149 * (J - 0.857)    
    return w

# get the rise velocity over a bunch of bubble sizes, and check it
d_vals = np.geomspace(5e-4,5e-3,101)
vq_vals = np.squeeze(np.array([rise_speed_Clift_contaminated(d) for d in d_vals]))

# get the drag coefficients and Reynolds numbers for each bubble size
CD_vals = 4 * d_vals * g / (3*vq_vals**2)
Req_vals = d_vals * vq_vals / nu

# bubble time scales
tau_p_visc = d_vals**2 * (C_M+gamma) / (18*nu)
x = CD_vals / (24./Req_vals)
tau_p_nonlin = tau_p_visc / x # SN -> Schiller-Naumann drag model

# Stokes number and v_s        
St = omega * tau_p_nonlin
v_s = vq_vals/c_p

# find the max Stokes enhancement
A = np.sqrt( (1-St**2*(1-beta))**2 + (St*(1-beta))**2 )
stokes_enhancement = A**2 / (1+v_s**2)
max_stokes_enhancement = np.nanmax(stokes_enhancement)

# plot the rise velocity and stokes enhancement as a function of d
fig_check,axs_check = plt.subplots(2,1,figsize=(4,7),sharex=True)
axs_check[0].semilogx(d_vals,vq_vals,)
axs_check[0].set_ylabel(r'$v_\mathrm{q}$ [m/s]')
axs_check[1].semilogx(d_vals,stokes_enhancement)
axs_check[1].set_ylabel(r'$v_{x,\mathrm{SD}} / u_{x,\mathrm{SD}}$')
axs_check[1].set_xlabel(r'$d$ [m]')
fig_check.tight_layout()

# plot Stokes drift and enhanced drift for four values of ak
ak_vals = [0.15,0.25,0.35,0.45]
for ak in ak_vals:
    
    color = get_color_ak(ak)
    
    # plot the Lagrangian Stokes drift
    zk = np.linspace(0,-1,1001)
    ux_SD_nondim = np.exp(2*zk) * ak**2
    ax.plot(ux_SD_nondim,zk,color=color)
    
    # shade the modified Lagrangian drift from DiBenedetto et al 2022
    vx_SD_nondim_max = ux_SD_nondim * max_stokes_enhancement
    ax.fill_betweenx(zk,ux_SD_nondim,vx_SD_nondim_max,color=color,alpha=0.5)
    
ax.set_xlabel(r'dimensionless horizontal transport velocity, $ \widetilde{\langle v_x \rangle} /  c_\mathrm{p} $')
ax.set_ylabel(r'dimensionless typical depth, $\widetilde{ \langle z \rangle} k_\mathrm{p}}$')    

'''
Distributions of vx_SD / ux_SD
'''

ax = axs[1]

# pdf of all the points
ax.axvline(df['vx_by_uxSD'].mean(),ls=':',color='r')
bins = np.geomspace(df['vx_by_uxSD'].min(),df['vx_by_uxSD'].max(),9)
x,y = get_hist(df['vx_by_uxSD'].values,bins=bins)
ax.loglog(x,y,'-o',color='r',label='entrainment',markerfacecolor='white',zorder=np.inf)

# pdf for specific ranges of ak
ak_bins = [0.15,0.35,0.45,0.6]
for ai in range(len(ak_bins)-1):
    df_use = df[df['ak']>=ak_bins[ai]]
    df_use = df_use[df_use['ak']<ak_bins[ai+1]]
    bins_clusters = np.geomspace(df_use['vx_by_uxSD'].min(),df_use['vx_by_uxSD'].max(),6)
    x,y = get_hist(df_use['vx_by_uxSD'].values,bins=bins_clusters)
    ax.loglog(x,y,color=get_color_ak(df_use['ak'].mean()))
    ax.axvline(df_use['vx_by_uxSD'].mean(),color=get_color_ak(df_use['ak'].mean()),ls=':',)
    
ax.set_xlabel(r'${\widetilde{\langle v_x \rangle}} / u_{x,\mathrm{SD}}$')
ax.set_ylabel(r'PDF')

fig.tight_layout()